# xgen_doc2chunk/core/processor/html_helper/__init__.py
"""HTML helper module for HTML file processing."""

from xgen_doc2chunk.core.processor.html_helper.html_file_converter import HTMLFileConverter

__all__ = ['HTMLFileConverter']

